/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: allocintuimessage.c,v 1.6 2019/08/18 07:18:26 piru Exp $
 
	Desc:
	Lang: english
*/
#include "intuition_intern.h"
#include <exec/exec.h>

/*****i***********************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH1(struct IntuiMessage *, AllocIntuiMessage,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 148, Intuition)

/*  FUNCTION
	Private to AROS: allocate an IntuiMessage. IntuiMessage->Window
	will be set to window by this function.
 
	INPUTS
	window - The window to which the IntuiMessage will be sent
 
	RESULT
	an allocated IntuiMessage structure. Must be freed with FreeIntuiMessage
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct IntuiMessage *msg;

	ASSERT_VALID_PTR(window);

	SANITY_CHECKR(window,0)

	ObtainSemaphore(&IBase->IDCMPCacheLock);

	msg = REMHEAD(&IBase->IDCMPCacheList);

	#ifdef SKINS
	if (!msg) msg = NewObjectA(IBase->imsgclass,NULL,NULL);
	#else
	if (!msg) msg = AllocPooled(IBase->IDCMPPool,sizeof(struct IntIntuiMessage));
	#endif

	ReleaseSemaphore(&IBase->IDCMPCacheLock);

	if (msg)
	{
		/* fill only the most important fields */

		msg->ExecMessage.mn_Node.ln_Type = NT_MESSAGE;
		msg->ExecMessage.mn_Length       = sizeof(struct ExtIntuiMessage);
		msg->ExecMessage.mn_ReplyPort    = window->WindowPort;

		msg->Class = 0;
		msg->IAddress = NULL;
		msg->IDCMPWindow = window;
		msg->SpecialLink = NULL;

		/* important one */
		((struct ExtIntuiMessage *)msg)->eim_TabletData = NULL;
	}

	DEBUG_ALLOCINTUIMESSAGE(dprintf("AllocIntuiMessage: Window 0x%lx Port 0x%lx Msg 0x%lx\n",
	                                window, window->WindowPort, msg));

	return msg;

	AROS_LIBFUNC_EXIT
}
